use tonic::transport::Channel;

pub mod error_server {
    tonic::include_proto!("error_server");
}

#[tokio::main]
async fn main() -> Result<(), anyhow::Error> {
    let env = env_logger::Env::default()
        .filter_or("LOG_LEVEL", "info")
        .write_style_or("LOG_STYLE", "never");
    env_logger::init_from_env(env);

    let port = std::env::var("PORT").unwrap_or_else(|_| "50051".to_string());
    let endpoint = format!("https://localhost:{port}");
    // log::info!("sending requests to {host}:{port}");
    let channel = Channel::from_shared(endpoint)?.connect().await?;
    log::info!("channel connected");

    let mut client = error_server::error_server_client::ErrorServerClient::new(channel);

    let request = tonic::Request::new(error_server::Request {
        text: "Hello World".to_string(),
    });

    match client.error_server(request).await {
        Err(status) => {
            // debug print the error showing that source is None.
            log::error!("received error: {:?}", status);
            Err(status.into())
        }
        Ok(response) => {
            log::info!("received response: {:?}", response.into_inner());
            Ok(())
        }
    }
}
