use std::sync::Arc;

use tonic::{transport::Server, Request, Response, Status};

pub mod error_server {
    tonic::include_proto!("error_server");
}

#[derive(Debug, Default)]
pub struct ErrorServerService {}

#[tonic::async_trait]
impl error_server::error_server_server::ErrorServer for ErrorServerService {
    /// Just return error
    async fn error_server(
        &self,
        _: Request<error_server::Request>,
    ) -> Result<Response<error_server::Response>, Status> {
        let source_error = std::io::Error::new(std::io::ErrorKind::Other, "oh no!");
        let mut status = tonic::Status::internal("error:");
        status.set_source(Arc::new(source_error));
        log::error!("sending error: {:?}", status);
        return Err(status);
    }
}

#[tokio::main]
async fn main() -> Result<(), Box<dyn std::error::Error>> {
    let env = env_logger::Env::default()
        .filter_or("LOG_LEVEL", "info")
        .write_style_or("LOG_STYLE", "never");
    env_logger::init_from_env(env);

    let port = std::env::var("PORT").unwrap_or_else(|_| "50051".to_string());
    let addr = std::net::SocketAddr::from(([0, 0, 0, 0], port.parse()?));
    let error_server = ErrorServerService::default();
    log::info!("serving on {addr}");

    Server::builder()
        .add_service(error_server::error_server_server::ErrorServerServer::new(
            error_server,
        ))
        .serve(addr)
        .await?;

    Ok(())
}
